import BaseLayer from '../core/BaseLayer';
import HeatMapModels from './models';
export default class HeatMapLayer extends BaseLayer {
  constructor(...args) {
    super(...args);
    this.name = 'HeatMapLayer';
  }

  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: 'number',
          minimum: 0,
          maximum: 1
        }
      }
    };
  }

  renderModels() {
    const shape = this.getModelType();

    if (shape === 'heatmap') {
      this.layerModel.render();
      return;
    }

    this.models.forEach(model => model.draw({
      uniforms: this.layerModel.getUninforms()
    }));
    return this;
  }

  buildModels() {
    const shape = this.getModelType();
    this.layerModel = new HeatMapModels[shape](this);
    this.models = this.layerModel.buildModels();
  }

  getModelType() {
    var _shapeAttribute$scale;

    const shapeAttribute = this.styleAttributeService.getLayerStyleAttribute('shape');
    const {
      shape3d
    } = this.getLayerConfig();
    const source = this.getSource();
    const sourceType = source.data.type;
    const shape = (shapeAttribute === null || shapeAttribute === void 0 ? void 0 : (_shapeAttribute$scale = shapeAttribute.scale) === null || _shapeAttribute$scale === void 0 ? void 0 : _shapeAttribute$scale.field) || 'heatmap';

    if (shape === 'heatmap' || shape === 'heatmap3d') {
      return 'heatmap';
    }

    if (sourceType === 'hexagon') {
      return (shape3d === null || shape3d === void 0 ? void 0 : shape3d.indexOf(shape)) === -1 ? 'hexagon' : 'grid3d';
    }

    if (sourceType === 'grid') {
      return (shape3d === null || shape3d === void 0 ? void 0 : shape3d.indexOf(shape)) === -1 ? 'grid' : 'grid3d';
    }

    return 'heatmap';
  }

}
//# sourceMappingURL=index.js.map