var _dec, _class;

import { AttributeType, gl } from '@antv/l7-core';
import { injectable } from 'inversify';
import { rgb2arr } from '../utils/color';

function encodePickingColor(featureIdx) {
  return [featureIdx + 1 & 255, featureIdx + 1 >> 8 & 255, featureIdx + 1 >> 8 >> 8 & 255];
}

const PickingStage = {
  NONE: 0.0,
  ENCODE: 1.0,
  HIGHLIGHT: 2.0
};
let PixelPickingPlugin = (_dec = injectable(), _dec(_class = class PixelPickingPlugin {
  apply(layer, {
    rendererService,
    styleAttributeService
  }) {
    layer.hooks.init.tap('PixelPickingPlugin', () => {
      const {
        enablePicking
      } = layer.getLayerConfig();
      styleAttributeService.registerStyleAttribute({
        name: 'pickingColor',
        type: AttributeType.Attribute,
        descriptor: {
          name: 'a_PickingColor',
          buffer: {
            data: [],
            type: gl.FLOAT
          },
          size: 3,
          update: (feature, featureIdx) => enablePicking ? encodePickingColor(featureIdx) : [0, 0, 0]
        }
      });
    });
    layer.hooks.beforePickingEncode.tap('PixelPickingPlugin', () => {
      const {
        enablePicking
      } = layer.getLayerConfig();

      if (enablePicking) {
        layer.models.forEach(model => model.addUniforms({
          u_PickingStage: PickingStage.ENCODE
        }));
      }
    });
    layer.hooks.afterPickingEncode.tap('PixelPickingPlugin', () => {
      const {
        enablePicking
      } = layer.getLayerConfig();

      if (enablePicking) {
        layer.models.forEach(model => model.addUniforms({
          u_PickingStage: PickingStage.NONE,
          u_PickingColor: [0, 0, 0],
          u_HighlightColor: [0, 0, 0, 0]
        }));
      }
    });
    layer.hooks.beforeHighlight.tap('PixelPickingPlugin', pickedColor => {
      const {
        highlightColor
      } = layer.getLayerConfig();
      const highlightColorInArray = typeof highlightColor === 'string' ? rgb2arr(highlightColor) : highlightColor || [1, 0, 0, 1];
      layer.models.forEach(model => model.addUniforms({
        u_PickingStage: PickingStage.HIGHLIGHT,
        u_PickingColor: pickedColor,
        u_HighlightColor: highlightColorInArray.map(c => c * 255)
      }));
    });
  }

}) || _class);
export { PixelPickingPlugin as default };
//# sourceMappingURL=PixelPickingPlugin.js.map