"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _isObject2 = _interopRequireDefault(require("lodash/isObject"));

var _isFunction2 = _interopRequireDefault(require("lodash/isFunction"));

var _l7Core = require("@antv/l7-core");

var _eventemitter = require("eventemitter3");

var _mergeJsonSchemas = _interopRequireDefault(require("merge-json-schemas"));

var _tapable = require("tapable");

var _MultiPassRendererPlugin = require("../plugins/MultiPassRendererPlugin");

var _schema = _interopRequireDefault(require("./schema"));

var _dec, _dec2, _dec3, _class, _descriptor, _descriptor2, _descriptor3, _temp;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _initializerDefineProperty(target, property, descriptor, context) { if (!descriptor) return; Object.defineProperty(target, property, { enumerable: descriptor.enumerable, configurable: descriptor.configurable, writable: descriptor.writable, value: descriptor.initializer ? descriptor.initializer.call(context) : void 0 }); }

function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) { var desc = {}; Object.keys(descriptor).forEach(function (key) { desc[key] = descriptor[key]; }); desc.enumerable = !!desc.enumerable; desc.configurable = !!desc.configurable; if ('value' in desc || desc.initializer) { desc.writable = true; } desc = decorators.slice().reverse().reduce(function (desc, decorator) { return decorator(target, property, desc) || desc; }, desc); if (context && desc.initializer !== void 0) { desc.value = desc.initializer ? desc.initializer.call(context) : void 0; desc.initializer = undefined; } if (desc.initializer === void 0) { Object.defineProperty(target, property, desc); desc = null; } return desc; }

function _initializerWarningHelper(descriptor, context) { throw new Error('Decorating class property failed. Please ensure that ' + 'proposal-class-properties is enabled and runs after the decorators transform.'); }

let layerIdCounter = 0;
let BaseLayer = (_dec = (0, _l7Core.lazyInject)(_l7Core.TYPES.ILogService), _dec2 = (0, _l7Core.lazyInject)(_l7Core.TYPES.IGlobalConfigService), _dec3 = (0, _l7Core.lazyInject)(_l7Core.TYPES.IShaderModuleService), (_class = (_temp = class BaseLayer extends _eventemitter.EventEmitter {
  constructor(config) {
    super();
    this.id = `${layerIdCounter++}`;
    this.name = void 0;
    this.visible = true;
    this.zIndex = 0;
    this.minZoom = void 0;
    this.maxZoom = void 0;
    this.inited = false;
    this.hooks = {
      init: new _tapable.SyncBailHook(),
      beforeRender: new _tapable.SyncBailHook(),
      afterRender: new _tapable.SyncHook(),
      beforePickingEncode: new _tapable.SyncHook(),
      afterPickingEncode: new _tapable.SyncHook(),
      beforeHighlight: new _tapable.SyncHook(['pickedColor']),
      afterHighlight: new _tapable.SyncHook(),
      beforeDestroy: new _tapable.SyncHook(),
      afterDestroy: new _tapable.SyncHook()
    };
    this.models = [];
    this.multiPassRenderer = void 0;
    this.plugins = void 0;
    this.sourceOption = void 0;

    _initializerDefineProperty(this, "logger", _descriptor, this);

    _initializerDefineProperty(this, "configService", _descriptor2, this);

    _initializerDefineProperty(this, "shaderModuleService", _descriptor3, this);

    this.iconService = void 0;
    this.fontService = void 0;
    this.rendererService = void 0;
    this.layerService = void 0;
    this.interactionService = void 0;
    this.mapService = void 0;
    this.styleAttributeService = void 0;
    this.layerSource = void 0;
    this.postProcessingPassFactory = void 0;
    this.normalPassFactory = void 0;
    this.layerModel = void 0;
    this.animateOptions = {
      enable: false
    };
    this.container = void 0;
    this.encodedData = void 0;
    this.configSchema = void 0;
    this.rawConfig = void 0;
    this.pendingStyleAttributes = [];
    this.scaleOptions = {};
    this.rawConfig = config;
  }

  getLayerConfig() {
    return this.configService.getLayerConfig(this.id);
  }

  updateLayerConfig(configToUpdate) {
    const sceneId = this.container.get(_l7Core.TYPES.SceneID);
    this.configService.setLayerConfig(sceneId, this.id, { ...this.configService.getLayerConfig(this.id),
      ...configToUpdate
    });
  }

  setContainer(container) {
    this.container = container;
  }

  getContainer() {
    return this.container;
  }

  addPlugin(plugin) {
    this.plugins.push(plugin);
    return this;
  }

  init() {
    const sceneId = this.container.get(_l7Core.TYPES.SceneID);
    this.configService.setLayerConfig(sceneId, this.id, this.rawConfig);
    this.iconService = this.container.get(_l7Core.TYPES.IIconService);
    this.fontService = this.container.get(_l7Core.TYPES.IFontService);
    this.rendererService = this.container.get(_l7Core.TYPES.IRendererService);
    this.layerService = this.container.get(_l7Core.TYPES.ILayerService);
    this.interactionService = this.container.get(_l7Core.TYPES.IInteractionService);
    this.mapService = this.container.get(_l7Core.TYPES.IMapService);
    this.postProcessingPassFactory = this.container.get(_l7Core.TYPES.IFactoryPostProcessingPass);
    this.normalPassFactory = this.container.get(_l7Core.TYPES.IFactoryNormalPass);
    this.styleAttributeService = this.container.get(_l7Core.TYPES.IStyleAttributeService);
    this.multiPassRenderer = this.container.get(_l7Core.TYPES.IMultiPassRenderer);
    this.multiPassRenderer.setLayer(this);
    this.pendingStyleAttributes.forEach(({
      attributeName,
      attributeField,
      attributeValues,
      defaultName,
      updateOptions
    }) => {
      this.styleAttributeService.updateStyleAttribute(attributeName, {
        scale: {
          field: attributeField,
          ...this.splitValuesAndCallbackInAttribute(attributeValues, this.getLayerConfig()[defaultName || attributeName])
        }
      }, updateOptions);
    });
    this.pendingStyleAttributes = [];
    this.plugins = this.container.getAll(_l7Core.TYPES.ILayerPlugin);

    for (const plugin of this.plugins) {
      plugin.apply(this, {
        rendererService: this.rendererService,
        mapService: this.mapService,
        styleAttributeService: this.styleAttributeService,
        normalPassFactory: this.normalPassFactory,
        postProcessingPassFactory: this.postProcessingPassFactory
      });
    }

    this.hooks.init.call();
    this.buildModels();
    this.inited = true;
    this.emit('inited');
    return this;
  }

  color(field, values, updateOptions) {
    this.pendingStyleAttributes.push({
      attributeName: 'color',
      attributeField: field,
      attributeValues: values,
      defaultName: 'colors',
      updateOptions
    });
    return this;
  }

  size(field, values, updateOptions) {
    this.pendingStyleAttributes.push({
      attributeName: 'size',
      attributeField: field,
      attributeValues: values,
      updateOptions
    });
    return this;
  }

  shape(field, values, updateOptions) {
    this.pendingStyleAttributes.push({
      attributeName: 'shape',
      attributeField: field,
      attributeValues: values,
      updateOptions
    });
    return this;
  }

  label(field, values, updateOptions) {
    this.pendingStyleAttributes.push({
      attributeName: 'label',
      attributeField: field,
      attributeValues: values,
      updateOptions
    });
    return this;
  }

  animate(options) {
    this.animateOptions = options;
    return this;
  }

  source(data, options) {
    this.sourceOption = {
      data,
      options
    };
    return this;
  }

  style(options) {
    const {
      passes,
      ...rest
    } = options;

    if (passes) {
      (0, _MultiPassRendererPlugin.normalizePasses)(passes).forEach(pass => {
        const postProcessingPass = this.multiPassRenderer.getPostProcessor().getPostProcessingPassByName(pass[0]);

        if (postProcessingPass) {
          postProcessingPass.updateOptions(pass[1]);
        }
      });
    }

    this.rawConfig = { ...this.rawConfig,
      ...rest
    };

    if (this.container) {
      this.updateLayerConfig(this.rawConfig);
    }

    return this;
  }

  scale(field, cfg) {
    if ((0, _isObject2.default)(field)) {
      this.scaleOptions = { ...this.scaleOptions,
        ...field
      };
    } else {
      this.scaleOptions[field] = cfg;
    }

    return this;
  }

  render() {
    if (this.multiPassRenderer && this.multiPassRenderer.getRenderFlag()) {
      this.multiPassRenderer.render();
    } else {
      this.renderModels();
    }

    return this;
  }

  show() {
    this.updateLayerConfig({
      visible: true
    });
    this.layerService.renderLayers();
    return this;
  }

  hide() {
    this.updateLayerConfig({
      visible: false
    });
    this.layerService.renderLayers();
    return this;
  }

  setIndex(index) {
    this.zIndex = index;
    this.layerService.updateRenderOrder();
    return this;
  }

  isVisible() {
    const zoom = this.mapService.getZoom();
    const {
      visible,
      minZoom = -Infinity,
      maxZoom = Infinity
    } = this.getLayerConfig();
    return !!visible && zoom >= minZoom && zoom <= maxZoom;
  }

  setMinZoom(minZoom) {
    this.updateLayerConfig({
      minZoom
    });
    return this;
  }

  setMaxZoom(maxZoom) {
    this.updateLayerConfig({
      maxZoom
    });
    return this;
  }

  fitBounds() {
    const source = this.getSource();
    const extent = source.extent;
    this.mapService.fitBounds([[extent[0], extent[1]], [extent[2], extent[3]]]);
  }

  destroy() {
    this.hooks.beforeDestroy.call();
    this.styleAttributeService.clearAllAttributes();
    this.models.forEach(model => model.destroy());
    this.hooks.afterDestroy.call();
  }

  isDirty() {
    return !!(this.styleAttributeService.getLayerStyleAttributes() || []).filter(attribute => attribute.needRescale || attribute.needRemapping || attribute.needRegenerateVertices).length;
  }

  setSource(source) {
    this.layerSource = source;
  }

  getSource() {
    return this.layerSource;
  }

  getScaleOptions() {
    return this.scaleOptions;
  }

  setEncodedData(encodedData) {
    this.encodedData = encodedData;
  }

  getEncodedData() {
    return this.encodedData;
  }

  getConfigSchemaForValidation() {
    if (!this.configSchema) {
      this.configSchema = (0, _mergeJsonSchemas.default)([_schema.default, this.getConfigSchema()]);
    }

    return this.configSchema;
  }

  pick({
    x,
    y
  }) {
    this.interactionService.triggerHover({
      x,
      y
    });
  }

  buildLayerModel(options) {
    const {
      moduleName,
      vertexShader,
      fragmentShader,
      triangulation,
      ...rest
    } = options;
    this.shaderModuleService.registerModule(moduleName, {
      vs: vertexShader,
      fs: fragmentShader
    });
    const {
      vs,
      fs,
      uniforms
    } = this.shaderModuleService.getModule(moduleName);
    const {
      createModel
    } = this.rendererService;
    const {
      attributes,
      elements
    } = this.styleAttributeService.createAttributesAndIndices(this.encodedData, triangulation);
    return createModel({
      attributes,
      uniforms,
      fs,
      vs,
      elements,
      blend: {
        enable: true,
        func: {
          srcRGB: _l7Core.gl.SRC_ALPHA,
          srcAlpha: 1,
          dstRGB: _l7Core.gl.ONE_MINUS_SRC_ALPHA,
          dstAlpha: 1
        }
      },
      ...rest
    });
  }

  getConfigSchema() {
    throw new Error('Method not implemented.');
  }

  buildModels() {
    throw new Error('Method not implemented.');
  }

  renderModels() {
    throw new Error('Method not implemented.');
  }

  splitValuesAndCallbackInAttribute(valuesOrCallback, defaultValues) {
    return {
      values: (0, _isFunction2.default)(valuesOrCallback) ? undefined : valuesOrCallback || defaultValues,
      callback: (0, _isFunction2.default)(valuesOrCallback) ? valuesOrCallback : undefined
    };
  }

}, _temp), (_descriptor = _applyDecoratedDescriptor(_class.prototype, "logger", [_dec], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor2 = _applyDecoratedDescriptor(_class.prototype, "configService", [_dec2], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor3 = _applyDecoratedDescriptor(_class.prototype, "shaderModuleService", [_dec3], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
})), _class));
exports.default = BaseLayer;
//# sourceMappingURL=BaseLayer.js.map