"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.polygonPath = polygonPath;
exports.circle = circle;
exports.square = square;
exports.triangle = triangle;
exports.hexagon = hexagon;
exports.pentagon = pentagon;
exports.geometryShape = exports.ShapeType2D = exports.ShapeType3D = void 0;
let ShapeType3D;
exports.ShapeType3D = ShapeType3D;

(function (ShapeType3D) {
  ShapeType3D["CYLINDER"] = "cylinder";
  ShapeType3D["SQUARECOLUMN"] = "squareColumn";
  ShapeType3D["TRIANGLECOLUMN"] = "triangleColumn";
  ShapeType3D["HEXAGONCOLUMN"] = "hexagonColumn";
  ShapeType3D["PENTAGONCOLUMN"] = "pentagonColumn";
})(ShapeType3D || (exports.ShapeType3D = ShapeType3D = {}));

let ShapeType2D;
exports.ShapeType2D = ShapeType2D;

(function (ShapeType2D) {
  ShapeType2D["CIRCLE"] = "circle";
  ShapeType2D["SQUARE"] = "square";
  ShapeType2D["TRIANGLE"] = "triangle";
  ShapeType2D["HEXAGON"] = "hexagon";
  ShapeType2D["PENTAGON"] = "pentagon";
})(ShapeType2D || (exports.ShapeType2D = ShapeType2D = {}));

function polygonPath(pointCount, start = 0) {
  const step = Math.PI * 2 / pointCount;
  const line = [];

  for (let i = 0; i < pointCount; i++) {
    line.push(step * i + start * Math.PI / 12);
  }

  const path = line.map(t => {
    const x = Math.sin(t + Math.PI / 4);
    const y = Math.cos(t + Math.PI / 4);
    return [x, y, 0];
  });
  return path;
}

function circle() {
  return polygonPath(30);
}

function square() {
  return polygonPath(4);
}

function triangle() {
  return polygonPath(3);
}

function hexagon() {
  return polygonPath(6, 1);
}

function pentagon() {
  return polygonPath(5);
}

const geometryShape = {
  [ShapeType2D.CIRCLE]: circle,
  [ShapeType2D.HEXAGON]: hexagon,
  [ShapeType2D.TRIANGLE]: triangle,
  [ShapeType2D.SQUARE]: square,
  [ShapeType2D.PENTAGON]: pentagon,
  [ShapeType3D.CYLINDER]: circle,
  [ShapeType3D.HEXAGONCOLUMN]: hexagon,
  [ShapeType3D.TRIANGLECOLUMN]: triangle,
  [ShapeType3D.SQUARECOLUMN]: square,
  [ShapeType3D.PENTAGONCOLUMN]: pentagon
};
exports.geometryShape = geometryShape;
//# sourceMappingURL=Path.js.map