"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _BaseLayer = _interopRequireDefault(require("../core/BaseLayer"));

var _index = _interopRequireDefault(require("./models/index"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class PointLayer extends _BaseLayer.default {
  constructor(...args) {
    super(...args);
    this.name = 'PointLayer';
  }

  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: 'number',
          minimum: 0,
          maximum: 1
        }
      }
    };
  }

  renderModels() {
    this.models.forEach(model => model.draw({
      uniforms: this.layerModel.getUninforms()
    }));
    return this;
  }

  buildModels() {
    const modelType = this.getModelType();
    this.layerModel = new _index.default[modelType](this);
    this.models = this.layerModel.buildModels();
  }

  getModelType() {
    const layerData = this.getEncodedData();
    const {
      shape2d,
      shape3d
    } = this.getLayerConfig();
    const iconMap = this.iconService.getIconMap();
    const item = layerData.find(fe => {
      return fe.hasOwnProperty('shape');
    });

    if (!item) {
      return 'normal';
    } else {
      const shape = item.shape;

      if ((shape2d === null || shape2d === void 0 ? void 0 : shape2d.indexOf(shape)) !== -1) {
        return 'fill';
      }

      if ((shape3d === null || shape3d === void 0 ? void 0 : shape3d.indexOf(shape)) !== -1) {
        return 'extrude';
      }

      if (iconMap.hasOwnProperty(shape)) {
        return 'image';
      }

      return 'text';
    }
  }

}

exports.default = PointLayer;
//# sourceMappingURL=index.js.map