"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RasterTriangulation = RasterTriangulation;

var _martini = _interopRequireDefault(require("@mapbox/martini"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function RasterTriangulation(parserData) {
  const {
    coordinates,
    data,
    min,
    max,
    width,
    height
  } = parserData;
  const maxlength = Math.max(width, height);
  const gridSize = Math.pow(2, Math.ceil(Math.log2(maxlength))) + 1;
  const terrain = new Float32Array(gridSize * gridSize);

  for (let i = 0; i < width; i++) {
    for (let j = 0; j < height; j++) {
      terrain[i * gridSize + j] = data[i * width + j];
    }
  }

  const martini = new _martini.default(gridSize);
  const tile = martini.createTile(terrain);
  const mesh = tile.getMesh(gridSize / 2);
  return {
    vertices: Array.from(mesh.vertices),
    indices: Array.from(mesh.triangles),
    size: 2
  };
}
//# sourceMappingURL=triangulation.js.map